/*
 * This program is distributed under the terms of the GPL v2.0 or later
 * Download the GNU Public License (GPL) from www.gnu.org
 *  Support the Free Software Foundation.
*/

#pragma pack(2)
#include <Pilot.h>

#include "tag.h"
#include "doclist.h"
#define MAXLEN_TAGNUM 10

static UInt GetNumPart(CharPtr dp, CharPtr sp, UInt max)
{
   CharPtr p;
   UInt len;
   p = dp;
   len = 0;
   while (len < max && *sp >= '0' && *sp <= '9') {
      *p = *sp;
      p++;
      sp++;
      len++;
   }
   *p = '\0';
   if (*sp == ':' || *sp == ',') {
      return len;
   } else {
      *dp = '\0';
      return 0;
   }
}


static UInt GetPart(CharPtr dp, CharPtr sp, UInt max)
{
   CharPtr p;
   UInt len;
   len = 0;
   p = dp;
   while (len < max && *sp != '\0' && *sp != ':' && *sp != '\n') {
      *p = *sp;
      p++;
      sp++;
      len++;
   }
   *p = '\0';
   if (*sp == ':') {
      return len;
   } else {
      *dp = '\0';
      return 0;
   }
}

/*
 * <doc>:<line>:<any><nl>
 * <doc>:<page>,<inspos>:<any><nl>
 * <doc>:[/|^]<find>[/|^]:<any><nl>
 * <line>:<any><nl>
 * <page>,<inspos>:<any><nl>
 * [/|^]<find>[/|^]:<any><nl>
 * :[/|^]<find>[/|^]:<any><nl>
 */
static void GetTag(CharPtr textPtr, UInt sPos,
		   CharPtr name, CharPtr num1, CharPtr num2)
{
   CharPtr p;
   CharPtr p1;
   Boolean find;
   UInt len;
   UInt step;

   name[0] = '\0';
   num1[0] = '\0';
   num2[0] = '\0';
   step = 0;

   p = textPtr + sPos;
   if (*p == '\n') {
      return;
   }
   while (sPos != -1 && *p != '\n') {
      sPos--;
      p--;
   }
   if (sPos == -1 || *p == '\n') {
      sPos++;
      p++;
   }
   if (*p == ':') {
      name[0] = ':';
      name[1] = '\0';
      p++;
      step = 1;			/* next is num1 */
   }

   len = GetNumPart(num1, p, MAXLEN_TAGNUM - 1);
   if (len > 0) {
      p = p + len;
      step = 2;			/* next is num2 */
   }
   if (step == 0) {		/* doc file or find strings(num1) */
      if (*p == '/' || *p == '^') {	/* find strings */
	 len = GetPart(num1, p, dmDBNameLength - 1);
	 step = 9;		/* next is end */
      } else {
	 len = GetPart(name, p, dmDBNameLength - 1);
	 if (len > 0) {
	    p = p + len + 1;
	    step = 1;		/* next is num1 */
	 }
      }
   }

   if (step == 1) {
      if (*p == '/' || *p == '^') {	/* find strings */
	 len = GetPart(num1, p, dmDBNameLength - 1);
	 if (len > 0) {
	    p = p + len;
	    step = 9;		/* next is end */
	 }
      } else {
	 len = GetNumPart(num1, p, MAXLEN_TAGNUM - 1);
	 if (len > 0) {
	    p = p + len;
	    step = 2;		/* next is num2 */
	 }
      }
   }


   if (step == 2) {
      if (*p == ',') {		/* num2 */
	 p++;
	 len = GetNumPart(num2, p, MAXLEN_TAGNUM - 1);
      }
   }
}

static void MakeTaginfo(CharPtr name, CharPtr num1, CharPtr num2)
{
   UInt page;
   UInt pos;
   UInt no;
   Boolean find;
   find = false;
   goInfo.page = 0;
   if (num1[0] == '\0') {
      return;
   }

   if (num1[0] == '/' || num1[0] == '^') {
      CharPtr p;
      int sl;
      p = num1;
      sl = StrLen(p);
      if (sl > sizeof(goInfo.findStr) - 1) {
	 sl = sizeof(goInfo.findStr) - 1;
      }
      if (sl <= 2) {
	 return;
      }
      if (*p == '^') {
	 *p = '\n';
      } else {
	 p = p + 1;
	 sl = sl - 1;
      }
      sl = sl - 1;
      if (*(p + sl) == '^') {
	 *(p + sl) = '\n';
	 sl++;
      } else if (*(p + sl) == '/') {
	 // *(p+sl) = '\0';
      } else {
	 return;
      }
      *(p + sl) = '\0';

      StrCopy(goInfo.findStr, p);
      page = 1;
      pos = 0;
      find = true;
   } else {
      page = StrAToI(num1);
      if (page > 0) {
	 find = true;
	 pos = StrAToI(num2);
      }
   }

   if (find) {
      if (name[0] != '\0') {
	 if (name[0] == ':') {
	    no = -3;
	 } else {
	    no = getDocPos(name);
	 }
      } else {
	 no = -2;
      }
      goInfo.index = no;	/* -1:error, -2:current, -3:last, n:xx */
      if (*num2 == '\0') {
	 goInfo.line = page;
	 goInfo.page = 1;
#if TEST
	 FrmCustomAlert(alertInfo, "line:", num1, " ");
	 FrmCustomAlert(alertInfo, " - line:", goInfo.findStr, " ");
#endif
      } else {
	 goInfo.page = page;
	 goInfo.insPos = pos;
#if TEST
	 FrmCustomAlert(alertInfo, num1, " : ", num2);
#endif
      }
   }
}

#if dmDBNameLength < MAXLEN_TAGNUM
#error "dmDBNameLength < MAXLEN_TAGNUM"
#endif


void SearchTag(FieldPtr fld)
{
   VoidHand textHand;
   VoidPtr textPtr;
   char num1[dmDBNameLength];
   char num2[MAXLEN_TAGNUM];
   char name[dmDBNameLength];	/* CategoryTruncateName */

   UInt sPos;
   UInt len;

   textHand = (VoidHand) FldGetTextHandle(fld);
   textPtr = MemHandleLock(textHand);
   sPos = FldGetInsPtPosition(fld);

   GetTag(textPtr, sPos, name, num1, num2);
#if TEST
   FrmCustomAlert(alertInfo, "name:", name, " ");
   FrmCustomAlert(alertInfo, "num1:", num1, " ");
   FrmCustomAlert(alertInfo, "num2:", num2, " ");
#endif

   MemPtrUnlock(textPtr);
   MakeTaginfo(name, num1, num2);
}
